import pandas as pd
from pydantic import BaseModel
from typing import List
from aryaxai.client.client import APIClient
from aryaxai.common.enums import UserRole
from aryaxai.common.xai_uris import (
    CREATE_PROJECT_URI,
    GET_WORKSPACES_URI,
    UPDATE_WORKSPACE_URI,
    GET_NOTIFICATIONS_URI,
    CLEAR_NOTIFICATIONS_URI,
)
from aryaxai.core.project import Project


class Workspace(BaseModel):
    """Class to work with AryaXAI workspaces"""

    created_by: str
    user_workspace_name: str
    workspace_name: str
    created_at: str

    __api_client: APIClient

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.__api_client = kwargs.get("api_client")

    def rename_workspace(self, new_workspace_name: str) -> str:
        """rename the current workspace to new name

        :param new_workspace_name: name for the workspace to be renamed to
        :return: response
        """
        payload = {
            "workspace_name": self.workspace_name,
            "modify_req": {
                "rename_workspace": new_workspace_name,
            },
        }
        res = self.__api_client.post(UPDATE_WORKSPACE_URI, payload)
        self.user_workspace_name = new_workspace_name
        return res.get("details")

    def delete_workspace(self) -> str:
        """deletes the current workspace
        :return: response
        """
        payload = {
            "workspace_name": self.workspace_name,
            "modify_req": {"delete_workspace": self.user_workspace_name},
        }
        res = self.__api_client.post(UPDATE_WORKSPACE_URI, payload)
        return res.get("details")

    def add_user_to_workspace(self, email: str, role: UserRole) -> str:
        """adds user to current workspace

        :param email: user email
        :param role: user role ["admin", "user"]
        :return: response
        """
        payload = {
            "workspace_name": self.workspace_name,
            "modify_req": {
                "add_user_workspace": {
                    "email": email,
                    "role": role,
                },
            },
        }
        res = self.__api_client.post(UPDATE_WORKSPACE_URI, payload)
        return res.get("details")

    def remove_user_from_workspace(self, email: str) -> str:
        """removes user from the current workspace

        :param email: user email
        :return: response
        """
        payload = {
            "workspace_name": self.workspace_name,
            "modify_req": {
                "remove_user_workspace": email,
            },
        }
        res = self.__api_client.post(UPDATE_WORKSPACE_URI, payload)
        return res.get("details")

    def update_user_access_for_workspace(self, email: str, role: UserRole) -> str:
        """update the user access for the workspace

        :param email: user email
        :param role: new user role ["admin", "user"]
        :return: _description_
        """
        payload = {
            "workspace_name": self.workspace_name,
            "modify_req": {
                "update_user_workspace": {
                    "email": email,
                    "role": role,
                }
            },
        }
        res = self.__api_client.post(UPDATE_WORKSPACE_URI, payload)
        return res.get("details")

    def projects(self) -> pd.DataFrame:
        """get user projects for this Workspace

        :return: Projects details dataframe
        """
        workspaces = self.__api_client.get(GET_WORKSPACES_URI)
        current_workspace = next(
            filter(
                lambda workspace: workspace["workspace_name"] == self.workspace_name,
                workspaces["details"],
            )
        )
        projects_df = pd.DataFrame(
            current_workspace["projects"],
            columns=[
                "user_project_name",
                "access_type",
                "created_by",
                "created_at",
                "updated_at",
            ],
        )
        return projects_df

    def project(self, project_name: str) -> Project:
        """Select specific project

        :param project_name: Name of the project
        :return: Project
        """
        workspaces = self.__api_client.get(GET_WORKSPACES_URI)
        current_workspace = next(
            filter(
                lambda workspace: workspace["workspace_name"] == self.workspace_name,
                workspaces["details"],
            )
        )
        projects = [
            Project(api_client=self.__api_client, **project)
            for project in current_workspace["projects"]
        ]

        project = next(
            filter(lambda project: project.user_project_name == project_name, projects),
            None,
        )

        if project is None:
            raise Exception("Project Not Found")

        return project

    def create_project(self, project_name: str) -> Project:
        """creates new project in the current workspace

        :param project_name: name for the project
        :return: response
        """
        payload = {
            "project_name": project_name,
            "workspace_name": self.workspace_name,
        }
        res = self.__api_client.post(CREATE_PROJECT_URI, payload)

        if not res["success"]:
            raise Exception(res["details"])

        project = Project(api_client=self.__api_client, **res["details"])

        return project

    def get_notifications(self) -> pd.DataFrame:
        """get user workspace notifications

        :return: DataFrame
        """
        url = f"{GET_NOTIFICATIONS_URI}?workspace_name={self.workspace_name}"

        res = self.__api_client.get(url)

        if not res["success"]:
            raise Exception("Error while getting workspace notifications.")

        notifications = res["details"]

        if not notifications:
            return "No notifications found."

        return pd.DataFrame(notifications).reindex(
            columns=["project_name", "message", "time"]
        )

    def clear_notifications(self) -> str:
        """clear user workspace notifications

        :raises Exception: _description_
        :return: str
        """
        url = f"{CLEAR_NOTIFICATIONS_URI}?workspace_name={self.workspace_name}"

        res = self.__api_client.post(url)

        if not res["success"]:
            raise Exception("Error while clearing workspace notifications.")

        return res["details"]

    def __print__(self) -> str:
        return f"Workspace(user_workspace_name='{self.user_workspace_name}', created_by='{self.created_by}', created_at='{self.created_at}')"

    def __str__(self) -> str:
        return self.__print__()

    def __repr__(self) -> str:
        return self.__print__()



from typing import List, Optional, Callable


class Validate:
    @staticmethod
    def string(field_name: str, value: str):
        if not (value and value.strip()):
            raise Exception(f"{field_name} is not valid, cannot be empty")

    @staticmethod
    def list(field_name: str, value: List):
        if not value:
            raise Exception(f"{field_name} is required, cannot be empty list")

    @staticmethod
    def value_against_list(
        field_name: str,
        value: str | List,
        valid_list: List,
        handle_error: Optional[Callable] = None,
    ):
        try:
            if isinstance(value, str):
                if value not in valid_list:
                    raise Exception(
                        f"{value} is not valid {field_name}, select from \n{valid_list}"
                    )
            if isinstance(value, List):
                Validate.list(field_name, value)
                for val in value:
                    if val not in valid_list:
                        raise Exception(
                            f"{val} is not a valid {field_name}, pick valid {field_name} from \n{valid_list}"
                        )
        except Exception as e:
            if handle_error:
                handle_error()
            raise e

    @staticmethod
    def check_for_missing_keys(value: dict, required_keys: list):
        missing_keys = [key for key in required_keys if key not in value]

        if missing_keys:
            raise Exception(f"Keys not present: {missing_keys}")

    @staticmethod
    def validate_date_feature_val(payload: dict, all_datetime_features: List[str]):
        if payload.get("date_feature", None):
            if payload["date_feature"] not in all_datetime_features:
                raise Exception(
                    f"{payload} is not a valid date_feature. Pick a valid payload from {all_datetime_features}."
                )

            if not payload["baseline_date"]:
                raise Exception(
                    "baseline_date is required when date_feature is passed."
                )
            if not payload["current_date"]:
                raise Exception("current_date is required when date_feature is passed.")

            if (
                payload["baseline_date"]["start_date"]
                > payload["baseline_date"]["end_date"]
            ):
                raise Exception(
                    "start_date of baseline_date should be less than end_date."
                )
            if (
                payload["current_date"]["start_date"]
                > payload["current_date"]["end_date"]
            ):
                raise Exception(
                    "start_date of current_date should be less than end_date."
                )



from datetime import datetime
from typing import Callable, Optional
from aryaxai.client.client import APIClient
from IPython.display import display, HTML

from aryaxai.common.xai_uris import POLL_EVENTS

def parse_float(s):
    """parse float from string, return None if not possible

    :param s: string to parse
    :return: float or None
    """
    try:
        return float(s)
    except ValueError:
        return None

def parse_datetime(s, format='%Y-%m-%d %H:%M:%S'):
    """Parse datetime from string, return None if not possible

    :param s: string to parse
    :param format: format string for datetime parsing
    :return: datetime or None
    """
    try:
        return datetime.strptime(s, format)
    except ValueError:
        return None

def pretty_date(date: str) -> str:
    """return date in format dd-mm-YYYY HH:MM:SS

    :param date: str datetime
    :return: pretty datetime
    """
    try:
        datetime_obj = datetime.strptime(date, '%Y-%m-%dT%H:%M:%S.%f')
    except ValueError:
        try:
            datetime_obj = datetime.strptime(date, '%Y-%m-%d %H:%M:%S.%f')
        except ValueError:
            print("Date format invalid.")

    return datetime_obj.strftime('%d-%m-%Y %H:%M:%S')

def poll_events(
    api_client: APIClient,
    project_name: str,
    event_id: str,
    handle_failed_event: Optional[Callable] = None,
    progress_message: str = "progress",
):
    last_message = ""
    log_length = 0
    progress = 0

    for event in api_client.stream(
        f"{POLL_EVENTS}?project_name={project_name}&event_id={event_id}"
    ):
        details = event.get("details")

        if not event.get("success"):
            raise Exception(details)
        if details.get("logs"):
            print(details.get("logs")[log_length:])
            log_length = len(details.get("logs"))
        if details.get("message") != last_message:
            last_message = details.get("message")
            print(f"{details.get('message')}")
        if details.get("progress"):
            if details.get("progress") != progress:
                progress = details.get("progress")
                print(f"{progress_message}: {progress}%")
            # display(HTML(f"<progress style='width:100%' value='{progress}' max='100'></progress>"))
        if details.get("status") == "failed":
            if handle_failed_event:
                handle_failed_event()
            raise Exception(details.get("message"))